--=====================================================================================================================
--== Vga Converter pro Sharp MZ-800 (pro verzi hardware 1)						      DL2011 ==
--=====================================================================================================================
--== verze h1.f2.14											 	     ==
--=====================================================================================================================
--== Zdrojovy kod je mi uplne volny :-))									     ==
--== Veskere sireni a pozmenovani je povoleno									     ==
--=====================================================================================================================

library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_arith.ALL;
use IEEE.std_logic_unsigned.ALL;
entity VGACONV is port (
-----------------------------------------------------------------------------------------------------------------------
-- vnejsi piny pro pripojeni k Sharp MZ-800 ---------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
MZCLK			: in std_logic;				--hodiny 17,734475 MHz
MZRGBI			: in std_logic_vector(3 downto 0);	--vstup od MZ barvy (I=3,B=2,G=1,R=0)
MZHSYNC			: in std_logic;				--vstup od MZ horizontalni synchronizace
MZVSYNC			: in std_logic;				--vstup od MZ vertikalni synchronizace
-----------------------------------------------------------------------------------------------------------------------
-- vnejsi piny pro pripojeni RAM --------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
RAMADR			: out std_logic_vector(17 downto 0);	--adresa k RAM
RAMDAT			: inout std_logic_vector(15 downto 0);	--datova sbernice RAM
RAMWRT			: out std_logic;			--povoleni zapisu do RAM
-----------------------------------------------------------------------------------------------------------------------
-- vnejsi piny pro pripojeni k VGA ------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
VGACLK			: in std_logic;				--hodiny 40MHz
VGARED			: out std_logic;			--vga vystup cervena slozka
VGAGREEN		: out std_logic;			--vga vystup zelena slozka
VGABLUE			: out std_logic;			--vga vystup modra slozka
VGAINTENSITY		: out std_logic;			--vga vystup intenzita
VGAHSYNC		: out std_logic;			--vga vystup horizontalni synchronizace
VGAVSYNC		: out std_logic);			--vga vystup vertikalni synchronizace
-----------------------------------------------------------------------------------------------------------------------
end VGACONV;
architecture rtl of VGACONV is
-----------------------------------------------------------------------------------------------------------------------
-- vnitrni signaly pro vstupni cast -----------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
signal	MZHOR		: std_logic_vector(9 downto 0);		--horizontalni citac obrazu MZ800
signal	MZVER		: std_logic_vector(8 downto 0);		--vertikalni citac obrazu MZ800
signal	MZHSYN1		: std_logic;				--pamet stavu horizontalni synchronizace
signal	MZVSYN1		: std_logic;				--pamet stavu vertikalni synchronizace
signal	PIX3		: std_logic_vector(11 downto 0);	--pomocna pamet barvy 3 pixelu
signal	INRAM		: std_logic_vector(15 downto 0);	--pamet barvy 4 pixelu (vystup do ram)
signal	MZPAGE		: std_logic;				--vyber stranky pameti pro zapis
-----------------------------------------------------------------------------------------------------------------------
-- vnitrni signaly pro VGA cast ---------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
signal	VGAHOR		: std_logic_vector(10 downto 0);	--horizontalni citac
signal	VGAVER		: std_logic_vector(9 downto 0);		--vertikalni citac
signal	VGAENA		: std_logic;				--povoleni zobrazovani
signal	VGAENV		: std_logic;				--povoleni zobrazovani od vertikalniho citace
signal	VGAPAGE		: std_logic;				--vyber stranky pameti pro cteni
signal	OUTRAM		: std_logic_vector(15 downto 0);	--register prijatych dat z RAM
-----------------------------------------------------------------------------------------------------------------------
-- vnitrni signaly pro zapis do pameti RAM ----------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
signal	WRTREQ		: std_logic;				--pozadavek zapisu (vstupni cast - MZ)
signal  WRREQ2		: std_logic_vector(1 downto 0);		--register pozadavku zapisu (pro synchronizaci zapisu)
signal	WRTACK		: std_logic;				--pozadavek zapisu (VGA cast)
signal	RAMWR1		: std_logic;				--interni signal /WR pro pamet RAM
-----------------------------------------------------------------------------------------------------------------------
begin
-----------------------------------------------------------------------------------------------------------------------
--citace a synchronizace dat na vstupni casti -------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
process (MZCLK) begin
    if MZCLK'event and MZCLK='1' then
	MZHSYN1 <= MZHSYNC;					--pamet stavu MZHSYNC
	MZVSYN1 <= MZVSYNC;					--pamet stavu MZVSYNC
	WRTREQ <= '0';						--pozadavek o zapis je defaultne neaktivni
	if MZHSYN1='0' and MZHSYNC='1' then			--detekovan konec aktivity MZHSYNC
	    MZHOR <= "1101001001";				--inicializace horizontalniho citace (-/+ obraz vlevo/vpravo)
	    MZVER <= MZVER + 1;					--vertikalni citac +1
	else
	    MZHOR(1 downto 0) <= MZHOR(1 downto 0) +'1';	--citac pixelu +1
	    if MZHOR(1 downto 0)=3 then
		INRAM <= MZRGBI & PIX3;				--registruj vsechny ctyri pixely
		MZHOR(9 downto 2) <= MZHOR(9 downto 2) +'1';	--nastav citac na dalsi ctverici pixelu
		WRTREQ <= '1';					--nastav pozadavek o zapis
	    else
		PIX3 <= MZRGBI & PIX3(11 downto 4);		--registruj pixel
	    end if;
	end if;
	if MZVSYN1='0' and MZVSYNC='1' then			--detekovana nabezna hrana MZVSYNC
	    MZVER <= "111110001";				--inicializace vertikalniho citace (-/+ obraz nahoru/dolu)
	    MZPAGE <= not MZPAGE;				--preklopeni do druhe poloviny pameti
	end if;
    end if;
end process;
-----------------------------------------------------------------------------------------------------------------------
process (VGACLK) begin
    if VGACLK'event and VGACLK='0' then
-----------------------------------------------------------------------------------------------------------------------
-- detekce pozadavku o zapis do pameti RAM od vstupni casti -----------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
	WRREQ2 <= WRTREQ & WRREQ2(1);				--registruj pozadavek o zapis ze vstupni casti
	if WRREQ2=2 then					--detekovan pozadavek o zapis ze vstupni casti
	    WRTACK <= '1';					--nastav pozadavek o zapis
	end if;
-----------------------------------------------------------------------------------------------------------------------
-- generovani signalu WR pro pamet RAM --------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
	RAMWR1 <= '1';						--zapis je defaultne neaktivni
	if WRTACK='1' and VGAHOR(1 downto 0)/=2 then		--pokud je pozadavek zapisu, ktery je mozno vykonat
	    RAMWR1 <= '0';					--nastav vnitrni signal /WR
	    WRTACK <= '0';					--zrus pozadavek zapisu
	end if;
-----------------------------------------------------------------------------------------------------------------------
-- VGA - citani, nulovani horizontalniho a vertikalniho citace --------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
	if VGAHOR=1055 then					--pokud jsme docitali sloupce na radku
	    VGAHOR <= (others=>'0');				--nulovani horizontalniho citace
	    if VGAVER=627 then					--pokud jsme docitali radky na strance
		VGAVER <= (others=>'0');			--nulovani vertikalniho citace
		VGAPAGE <= not MZPAGE;				--preklopeni do stranky pameti do ktere MZ nezapisuje
	    else
		VGAVER <= VGAVER + '1';				--vertikalni citac +1
	    end if;
	else
	    VGAHOR <= VGAHOR + '1';				--horizontalni citac +1
	end if;
-----------------------------------------------------------------------------------------------------------------------
-- VGA - generovani vertikalni synchronizace --------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
	if VGAVER=0 then
	    VGAENV <= '1';					--povoleni zobrazeni
	elsif VGAVER=600 then
	    VGAENV <= '0';					--zakazani zobrazeni
	elsif VGAVER=601 then
	    VGAVSYNC <= '1';					--zacatek vertikalni synchronizace
	elsif VGAVER=605 then
	    VGAVSYNC <= '0';					--konec vertikalni synchronizace
	end if;
-----------------------------------------------------------------------------------------------------------------------
-- VGA - generovani horizontalni synchronizace ------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
	if VGAHOR=3 and VGAENV='1' then
	    VGAENA <= '1';					--povoleni zobrazovani
	elsif VGAHOR=803 then
	    VGAENA <= '0';					--zakazani zobrazovani
	elsif VGAHOR=843 then
	    VGAHSYNC <= '1';					--zacatek horizontalni synchronizace
	elsif VGAHOR=971 then
	    VGAHSYNC <= '0';					--konec horizontalni synchronizace
	end if;
-----------------------------------------------------------------------------------------------------------------------
-- VGA - cteni dat z pameti a jejich zapis do registru, rotace registru -----------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
	if VGAHOR(1 downto 0)=3 then
	    OUTRAM <= RAMDAT;					--zapis prectenych dat z pameti do registru
	else
	    OUTRAM(11 downto 0) <= OUTRAM(15 downto 4);		--posunuti registru o ctyri bity = posun na dalsi pixel
	end if;
-----------------------------------------------------------------------------------------------------------------------
    end if;
end process;
-----------------------------------------------------------------------------------------------------------------------
-- VGA - generovani barvonosnych signalu ------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
VGARED <=	'0' when VGAENA='0' else OUTRAM(0);		--vystup vga RED
VGAGREEN <=	'0' when VGAENA='0' else OUTRAM(1);		--vystup vga GREEN
VGABLUE <=	'0' when VGAENA='0' else OUTRAM(2);		--vystup vga BLUE
VGAINTENSITY <=	'0' when VGAENA='0' else OUTRAM(3);		--vystup vga intenzita
-----------------------------------------------------------------------------------------------------------------------
-- napojeni pameti RAM ------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------
RAMADR <= MZPAGE & MZVER(8 downto 0) & MZHOR(9 downto 2) when RAMWR1='0' else
	  VGAPAGE & "000000101" & VGAHOR(9 downto 2) when VGAVER(9 downto 1)<5 else
	  VGAPAGE & "100100010" & VGAHOR(9 downto 2) when VGAVER(9 downto 1)>290 else
	  VGAPAGE & VGAVER(9 downto 1) & VGAHOR(9 downto 2);
RAMDAT <= INRAM when RAMWR1='0' else "ZZZZZZZZZZZZZZZZ";
RAMWRT <= RAMWR1;
-----------------------------------------------------------------------------------------------------------------------
end rtl;
